# DateTrait

import slogan, slogan.traits
import string, re

# monthShortNames lists names as included in log file lines, so does not
# need to be localised

monthShortNames= {
  'jan': '01', 'feb': '02', 'mar': '03', 'apr': '04', 'may': '05', 'jun': '06',
  'jul': '07', 'aug': '08', 'sep': '09', 'oct': '10', 'nov': '11', 'dec': '12',
}

# dateFormats is a list of possible formats the date could be in. Each regexp in
# it is checked in turn until one matches. This one then jumps to the top of the
# list so that if the next date format is the same (highly likely) the lookup
# will be quicker.

# For each format, a lambda-form function is given that takes the re.match
# object and returns a tuple (year, month, day) by re-ordering the groups in
# the regexp match. It also returns a flag which is set true if we are sure that
# the ordering is correct, or false if we don't know which way round month
# and day go, thanks to daftness on some old web servers' part.

dateFormats= [
  (
    re.compile('(\\d\\d\\d\\d)-(\\d\\d)-(\\d\\d)$'),
    lambda match: (match.group(1, 2, 3), slogan.true)
  ), (
    re.compile('(\\d\\d)/(\\d\\d)/(\\d\\d)$'),
    lambda match: (match.group(3, 2, 1), slogan.false)
  ), (
    re.compile('(\\d\\d)/([A-Za-z][A-Za-z][A-Za-z])/(\\d\\d\\d\\d)$'),
    lambda match: ((
      match.group(3),
      monthShortNames.get(string.lower(match.group(2)), '00'),
      match.group(1)),
      slogan.true)
  ), (
    re.compile('(\\d\\d\\d\\d)(\\d\\d)(\\d\\d)$'),
    lambda match: (match.group(1, 2, 3), slogan.true)
  ), (
    re.compile('(\\d\\d)(\\d\\d)(\\d\\d)$'),
    lambda match: (match.group(1, 2, 3), slogan.true)
  )
]

# DateTrait does the work of managing the regexps, correcting two-digit year
# values, guessing which way round the day and month go (by keeping a flag
# holding the previous guess, and changing it if it no longer fits the data, ie.
# there's a month greater than 12), and writing out dates in a readable form.

class Trait(slogan.traits.Trait):
  identity= 'date'
  extended= 'date'
  priority= 100
  size= [4, 2, 2]
  def __init__(self):
    self.title= slogan.language.date_title
    self.dmReverse= slogan.config['logDateUS']

  def read(self, logpart):
    for i in range(len(dateFormats)):
      dateMatch= dateFormats[i][0].match(logpart)
      if dateMatch!=None:
        dateFormats[:0]= dateFormats[i:i+1]
        dateFormats[i+1:i+2]= []
        (dateList, dmOrdered)= dateFormats[0][1](dateMatch)
        dateList= list(dateList)
        if not dmOrdered:
          try:
            if int(dateList[1])<13:
              dmOrdered= true
          except ValueError:
            pass
          if self.dmReverse:
            dateList[1], dateList[2]= dateList[2], dateList[1]
        try:
          if int(dateList[0])<100:
            if dateList[0]<slogan.config['y2kWindow']:
              dateList[0]=str(int(dateList[0])+slogan.config['y2kCentury'])
            else:
              dateList[0]=str(int(dateList[0])+slogan.config['y2kCentury']-100)
        except ValueError:
          pass
        return dateList
    return []

  def write(self, partlist):
    output= ''
    if len(partlist)>=3:
      output= output+partlist[2]+' '
    if len(partlist)>=2:
      output= output+slogan.language.date_months.get(partlist[1], slogan.language.date_months['00'])+' '
    if len(partlist)>=1:
      if partlist[0][:1]=='0':
        output= output+partlist[0][1:]
      else:
        output= output+partlist[0]
    return output
